// Mock dictionary for immediate feedback
const dummyDictionary = {
  physics:
    "The branch of science concerned with the nature and properties of matter and energy.",
  branch: "A division or subdivision of a subject or field of knowledge.",
  science:
    "The intellectual and practical activity encompassing the systematic study of structure and behavior.",
  nature: "The physical world and everything in it not made by humans.",
  property: "An attribute or quality of something.",
  matter: "Physical substance that occupies space and possesses mass.",
  energy:
    "The strength and vitality required for sustained physical or mental activity.",
  field: "A particular branch of study or sphere of activity or interest.",
  knowledge:
    "Facts, information, and skills acquired through experience or education.",
  // Add more entries as needed
};

// Get definition and recursive definitions for a word
async function getWordDefinitions(word, dictionary, seenWords = new Set()) {
  const normalizedWord = word
    .toLowerCase()
    .trim()
    .replace(/[^a-z]/g, "");

  // Skip if already processed to avoid infinite recursion
  if (seenWords.has(normalizedWord)) {
    return [];
  }

  seenWords.add(normalizedWord);
  const results = [];

  const definition = dictionary[normalizedWord] || "[undefined]";

  // If we have a definition, process its words recursively FIRST
  if (definition !== "[undefined]") {
    const definitionWords = definition
      .toLowerCase()
      .replace(/[^\w\s]/g, "")
      .split(/\s+/)
      .filter((w) => w.length > 3 && !seenWords.has(w)); // Process only meaningful words

    for (const defWord of definitionWords) {
      const subDefinitions = await getWordDefinitions(
        defWord,
        dictionary,
        seenWords,
      );
      results.push(...subDefinitions);
    }
  }

  // NOW add the current word's definition (after all its component words)
  results.push(`${normalizedWord} ${definition}`);

  return results;
}

// Function to load dictionary from JSON file
async function loadDictionary() {
  try {
    const response = await fetch("dictionary.json");
    if (!response.ok) {
      console.warn("Failed to load dictionary.json.");
      return null;
    }
    const dictionary = await response.json();
    console.log("Dictionary loaded successfully from file.");
    return dictionary;
  } catch (error) {
    console.warn("Error loading dictionary.json:", error);
    return null;
  }
}

async function getPretext(text, dictionary) {
  const chars = [...new Set(text.toLowerCase().replace(/[^a-z]/gi, ""))];
  const words = text.split(/\s+/).filter((w) => w);
  const sentences = text.split(/[.!?]+/).filter((s) => s.trim());

  const pretext = new Set();
  const processedDefinitions = new Set(); // Track which words we've already processed

  // Character-level analysis
  chars.forEach((c) => {
    pretext.add("binary");
    pretext.add("letter");
    pretext.add(/[a-z]/.test(c) ? "lowercase" : "uppercase");
    pretext.add("utf8");
    pretext.add(c);
  });

  // Word-level analysis
  for (const word of words) {
    pretext.add("font");
    pretext.add("word");
    if (/^[A-Z]/.test(word)) pretext.add("propernoun");

    // Only process definitions if we haven't already processed this word
    const normalizedWord = word
      .toLowerCase()
      .trim()
      .replace(/[^a-z]/g, "");
    if (!processedDefinitions.has(normalizedWord)) {
      // Get definitions for this word
      const wordDefinitions = await getWordDefinitions(
        normalizedWord,
        dictionary,
      );

      // Add all definitions to processed set to avoid duplication
      wordDefinitions.forEach((def) => {
        const definedWord = def.split(" ")[0];
        processedDefinitions.add(definedWord);
        pretext.add(def);
      });
    }
  }

  // Phrase-level analysis
  if (words.length > 1) {
    pretext.add("phrase");
    const fullText = text.toLowerCase();
    if (fullText.includes("hello world")) pretext.add("commonPhrase");
    if (fullText.match(/greeting|welcome|hello/i)) pretext.add("greeting");
  }

  // Sentence-level analysis
  sentences.forEach((sentence) => {
    pretext.add("sentence");
    if (sentence.split(/\s+/).length > 5) pretext.add("long-form");
  });

  return Array.from(pretext)
    .filter((x) => x)
    .join("\n");
}

document.addEventListener("DOMContentLoaded", async () => {
  const textarea = document.getElementById("content");
  const pretextOutput = document.getElementById("pretext");
  const parserCount = document.getElementById("parserCount");
  const runFullButton = document.getElementById("runFull");
  const loadingIndicator = document.createElement("div");

  // Set up loading indicator
  loadingIndicator.id = "loadingIndicator";
  loadingIndicator.style.display = "none";
  loadingIndicator.innerHTML = `
    <div style="position: fixed; top: 0; left: 0; right: 0; bottom: 0;
                background-color: rgba(0,0,0,0.5); display: flex;
                justify-content: center; align-items: center; z-index: 1000;">
      <div style="background-color: white; padding: 20px; border-radius: 5px; text-align: center;">
        <div style="margin-bottom: 10px;">Loading full dictionary and processing...</div>
        <div style="width: 50px; height: 50px; border: 5px solid #f3f3f3;
                    border-top: 5px solid #3498db; border-radius: 50%;
                    animation: spin 2s linear infinite; margin: 0 auto;"></div>
      </div>
    </div>
    <style>
      @keyframes spin {
        0% { transform: rotate(0deg); }
        100% { transform: rotate(360deg); }
      }
    </style>
  `;
  document.body.appendChild(loadingIndicator);

  // For immediate feedback while typing
  async function updateAnalysis() {
    const text = textarea.value;
    const pretext = await getPretext(text, dummyDictionary);
    pretextOutput.textContent = pretext;
    parserCount.textContent = pretext.split("\n").length - 1;
  }

  // For full analysis with complete dictionary
  async function runFullAnalysis() {
    const text = textarea.value;

    // Show loading indicator
    loadingIndicator.style.display = "block";

    try {
      // Load dictionary (only when needed)
      const fullDictionary = await loadDictionary();

      // Use full dictionary if loaded, otherwise fallback to dummy
      const dictionaryToUse = fullDictionary || dummyDictionary;

      // Process with the selected dictionary
      const pretext = await getPretext(text, dictionaryToUse);
      pretextOutput.textContent = pretext;
      parserCount.textContent = pretext.split("\n").length - 1;

      // Show a message if we had to fall back
      if (!fullDictionary) {
        console.warn(
          "Full dictionary could not be loaded. Using dummy dictionary instead.",
        );
        alert(
          "Could not load the full dictionary. Results shown are using the limited dictionary.",
        );
      }
    } catch (error) {
      console.error("Error in full analysis:", error);
      alert("An error occurred during processing. Please try again.");
    } finally {
      // Hide loading indicator
      loadingIndicator.style.display = "none";
    }
  }

  // Event listeners
  textarea.addEventListener("input", updateAnalysis);
  runFullButton.addEventListener("click", runFullAnalysis);

  // Initial analysis with dummy dictionary
  updateAnalysis();
});
